@@ -3,8 +3,8 @@ from contextlib import contextmanager
 import dbt.exceptions
 from dbt.adapters.base import Credentials
 from dbt.adapters.sql import SQLConnectionManager
-from dbt.contracts.connection import ConnectionState
-from dbt.logger import GLOBAL_LOGGER as logger
+from dbt.contracts.connection import ConnectionState, AdapterResponse
+from dbt.events import AdapterLogger
 from dbt.utils import DECIMALS
 from dbt.adapters.spark import __version__
 
@@ -24,12 +24,26 @@ from datetime import datetime
 import sqlparams
 
 from hologram.helpers import StrEnum
-from dataclasses import dataclass
-from typing import Optional
+from dataclasses import dataclass, field
+from typing import Any, Dict, Optional
+try:
+    from thrift.transport.TSSLSocket import TSSLSocket
+    import thrift
+    import ssl
+    import sasl
+    import thrift_sasl
+except ImportError:
+    TSSLSocket = None
+    thrift = None
+    ssl = None
+    sasl = None
+    thrift_sasl = None
 
 import base64
 import time
 
+logger = AdapterLogger("Spark")
+
 NUMBERS = DECIMALS + (int, float)
 
 
@@ -47,7 +61,6 @@ class SparkConnectionMethod(StrEnum):
 class SparkCredentials(Credentials):
     host: str
     method: SparkConnectionMethod
-    schema: str
     database: Optional[str]
     driver: Optional[str] = None
     cluster: Optional[str] = None
@@ -60,6 +73,16 @@ class SparkCredentials(Credentials):
     organization: str = '0'
     connect_retries: int = 0
     connect_timeout: int = 10
+    use_ssl: bool = False
+    server_side_parameters: Dict[str, Any] = field(default_factory=dict)
+    retry_all: bool = False
+
+    @classmethod
+    def __pre_deserialize__(cls, data):
+        data = super().__pre_deserialize__(data)
+        if 'database' not in data:
+            data['database'] = None
+        return data
 
     def __post_init__(self):
         # spark classifies database and schema as the same thing
@@ -75,13 +98,17 @@ class SparkCredentials(Credentials):
             )
         self.database = None
 
-        if self.method == SparkConnectionMethod.ODBC and pyodbc is None:
-            raise dbt.exceptions.RuntimeException(
-                f"{self.method} connection method requires "
-                "additional dependencies. \n"
-                "Install the additional required dependencies with "
-                "`pip install dbt-spark[ODBC]`"
-            )
+        if self.method == SparkConnectionMethod.ODBC:
+            try:
+                import pyodbc    # noqa: F401
+            except ImportError as e:
+                raise dbt.exceptions.RuntimeException(
+                    f"{self.method} connection method requires "
+                    "additional dependencies. \n"
+                    "Install the additional required dependencies with "
+                    "`pip install dbt-spark[ODBC]`\n\n"
+                    f"ImportError({e.msg})"
+                ) from e
 
         if (
             self.method == SparkConnectionMethod.ODBC and
@@ -110,6 +137,10 @@ class SparkCredentials(Credentials):
     def type(self):
         return 'spark'
 
+    @property
+    def unique_field(self):
+        return self.host
+
     def _connection_keys(self):
         return ('host', 'port', 'cluster',
                 'endpoint', 'schema', 'organization')
@@ -148,6 +179,7 @@ class PyhiveConnectionWrapper(object):
                 logger.debug(
                     "Exception while closing cursor: {}".format(exc)
                 )
+        self.handle.close()
 
     def rollback(self, *args, **kwargs):
         logger.debug("NotImplemented: rollback")
@@ -248,7 +280,7 @@ class SparkConnectionManager(SQLConnectionManager):
     SPARK_CLUSTER_HTTP_PATH = "/sql/protocolv1/o/{organization}/{cluster}"
     SPARK_SQL_ENDPOINT_HTTP_PATH = "/sql/1.0/endpoints/{endpoint}"
     SPARK_CONNECTION_URL = (
-        "https://{host}:{port}" + SPARK_CLUSTER_HTTP_PATH
+        "{host}:{port}" + SPARK_CLUSTER_HTTP_PATH
     )
 
     @contextmanager
@@ -273,8 +305,12 @@ class SparkConnectionManager(SQLConnectionManager):
         connection.handle.cancel()
 
     @classmethod
-    def get_response(cls, cursor):
-        return 'OK'
+    def get_response(cls, cursor) -> AdapterResponse:
+        # https://github.com/dbt-labs/dbt-spark/issues/142
+        message = 'OK'
+        return AdapterResponse(
+            _message=message
+        )
 
     # No transactions on Spark....
     def add_begin_query(self, *args, **kwargs):
@@ -314,8 +350,13 @@ class SparkConnectionManager(SQLConnectionManager):
                     cls.validate_creds(creds, ['token', 'host', 'port',
                                                'cluster', 'organization'])
 
+                    # Prepend https:// if it is missing
+                    host = creds.host
+                    if not host.startswith('https://'):
+                        host = 'https://' + creds.host
+
                     conn_url = cls.SPARK_CONNECTION_URL.format(
-                        host=creds.host,
+                        host=host,
                         port=creds.port,
                         organization=creds.organization,
                         cluster=creds.cluster
@@ -337,14 +378,22 @@ class SparkConnectionManager(SQLConnectionManager):
                     cls.validate_creds(creds,
                                        ['host', 'port', 'user', 'schema'])
 
-                    conn = hive.connect(host=creds.host,
-                                        port=creds.port,
-                                        username=creds.user,
-                                        auth=creds.auth,
-                                        kerberos_service_name=creds.kerberos_service_name)  # noqa
+                    if creds.use_ssl:
+                        transport = build_ssl_transport(
+                            host=creds.host,
+                            port=creds.port,
+                            username=creds.user,
+                            auth=creds.auth,
+                            kerberos_service_name=creds.kerberos_service_name)
+                        conn = hive.connect(thrift_transport=transport)
+                    else:
+                        conn = hive.connect(host=creds.host,
+                                            port=creds.port,
+                                            username=creds.user,
+                                            auth=creds.auth,
+                                            kerberos_service_name=creds.kerberos_service_name)  # noqa
                     handle = PyhiveConnectionWrapper(conn)
                 elif creds.method == SparkConnectionMethod.ODBC:
-                    http_path = None
                     if creds.cluster is not None:
                         required_fields = ['driver', 'host', 'port', 'token',
                                            'organization', 'cluster']
@@ -369,6 +418,12 @@ class SparkConnectionManager(SQLConnectionManager):
                     dbt_spark_version = __version__.version
                     user_agent_entry = f"fishtown-analytics-dbt-spark/{dbt_spark_version} (Databricks)"  # noqa
 
+                    # http://simba.wpengine.com/products/Spark/doc/ODBC_InstallGuide/unix/content/odbc/hi/configuring/serverside.htm
+                    ssp = {
+                        f"SSP_{k}": f"{{{v}}}"
+                        for k, v in creds.server_side_parameters.items()
+                    }
+
                     # https://www.simba.com/products/Spark/doc/v2/ODBC_InstallGuide/unix/content/odbc/options/driver.htm
                     connection_str = _build_odbc_connnection_string(
                         DRIVER=creds.driver,
@@ -382,6 +437,8 @@ class SparkConnectionManager(SQLConnectionManager):
                         ThriftTransport=2,
                         SSL=1,
                         UserAgentEntry=user_agent_entry,
+                        LCaseSspKeyName=0 if ssp else 1,
+                        **ssp,
                     )
 
                     conn = pyodbc.connect(connection_str, autocommit=True)
@@ -409,6 +466,16 @@ class SparkConnectionManager(SQLConnectionManager):
                     )
                     logger.warning(msg)
                     time.sleep(creds.connect_timeout)
+                elif creds.retry_all and creds.connect_retries > 0:
+                    msg = (
+                        f"Warning: {getattr(exc, 'message', 'No message')}, "
+                        f"retrying due to 'retry_all' configuration "
+                        f"set to true.\n\tRetrying in "
+                        f"{creds.connect_timeout} seconds "
+                        f"({i} of {creds.connect_retries})"
+                    )
+                    logger.warning(msg)
+                    time.sleep(creds.connect_timeout)
                 else:
                     raise dbt.exceptions.FailedToConnectException(
                         'failed to connect'
@@ -421,6 +488,49 @@ class SparkConnectionManager(SQLConnectionManager):
         return connection
 
 
+def build_ssl_transport(host, port, username, auth,
+                        kerberos_service_name, password=None):
+    transport = None
+    if port is None:
+        port = 10000
+    if auth is None:
+        auth = 'NONE'
+    socket = TSSLSocket(host, port, cert_reqs=ssl.CERT_NONE)
+    if auth == 'NOSASL':
+        # NOSASL corresponds to hive.server2.authentication=NOSASL
+        # in hive-site.xml
+        transport = thrift.transport.TTransport.TBufferedTransport(socket)
+    elif auth in ('LDAP', 'KERBEROS', 'NONE', 'CUSTOM'):
+        # Defer import so package dependency is optional
+        if auth == 'KERBEROS':
+            # KERBEROS mode in hive.server2.authentication is GSSAPI
+            # in sasl library
+            sasl_auth = 'GSSAPI'
+        else:
+            sasl_auth = 'PLAIN'
+            if password is None:
+                # Password doesn't matter in NONE mode, just needs
+                # to be nonempty.
+                password = 'x'
+
+        def sasl_factory():
+            sasl_client = sasl.Client()
+            sasl_client.setAttr('host', host)
+            if sasl_auth == 'GSSAPI':
+                sasl_client.setAttr('service', kerberos_service_name)
+            elif sasl_auth == 'PLAIN':
+                sasl_client.setAttr('username', username)
+                sasl_client.setAttr('password', password)
+            else:
+                raise AssertionError
+            sasl_client.init()
+            return sasl_client
+
+        transport = thrift_sasl.TSaslClientTransport(sasl_factory,
+                                                     sasl_auth, socket)
+    return transport
+
+
 def _is_retryable_error(exc: Exception) -> Optional[str]:
     message = getattr(exc, 'message', None)
     if message is None:
